@@ -1,8 +1,8 @@
 """
 This module implements the compiler pass to translate the tree of SQLnode 
-objects to a tree of SQLClause objects, which is almost like the actual 
-lexical structure of the SQL query. The SQLClause expression can then be 
-serialized into the SQL query string. 
+objects to a tree of SQLClause objects, which represents the lexical structure 
+of the SQL query. The SQLClause expression can then be serialized almost 
+directly into the SQL query string. 
 """
 
 from contextlib import contextmanager
@@ -28,7 +28,7 @@ class Assemblage:
     """Represents a partially constructed SQL query"""
 
     clause: Optional[SQLClause]  # An SQL subquery, with(out) SELECT args
-    cols: ALIAS_TO_CLAUSE  # TODO: SELECT args if required. Umm what?
+    cols: ALIAS_TO_CLAUSE  # SELECT args if required
     repl: NODE_TO_ALIAS  # mapping referenced nodes to column aliases
 
     def __init__(
@@ -42,8 +42,8 @@ class Assemblage:
         self.repl = repl if repl is not None else dict()
 
     def get_complete_select(self) -> SQLClause:
-        """Construct the SELECT clause to a partially assembled query and return.
-        NOTE: the clause attribute is not modified in place.
+        """Construct the SELECT clause to a partially assembled query if it didn't have one
+        and return.
         """
         clause = self.clause
         if not isinstance(clause, (SELECT, UNION)):
@@ -54,7 +54,6 @@ class Assemblage:
     def get_make_subs(self, alias: Optional[Symbol]) -> NODE_TO_CLAUSE:
         """Build and return a node-to-clause mapping assuming the assemblage will be
         extended/completed, if the input `alias` arg is None/not None.
-        TODO: what?
         """
         subs: NODE_TO_CLAUSE = dict()
         if alias is None:
@@ -77,8 +76,8 @@ class Assemblage:
 
 
 def cols_to_select_args(cols: ALIAS_TO_CLAUSE) -> list[SQLClause]:
-    """Pack the arguments to a SELECT clause; goes over all the columns
-    and assigns them aliases.
+    """Pack the arguments to a SELECT clause; this routine goes over all the
+    columns and assigns them aliases.
     """
     args = []
     for name, clause in cols.items():
@@ -105,7 +104,7 @@ def translates_to_repl_n_cols(
         counter[name] = counter.get(name, 0) + 1
         k = counter[name]
 
-        # TODO: this is a trouble site, until we implement equality for *all* SQLClause objects
+        # TODO: this is a trouble site, until we implement eq/hash for *all* SQLClause objects
         if k > 1 and (name, clause) in renames:
             # filters out duplicate columns
             name_p = renames[(name, clause)]
@@ -143,7 +142,6 @@ class CTEAssemblage:
     name: Symbol
     schema: Optional[Symbol]
     materialized: Optional[bool]
-    external: bool
 
     def __init__(
         self,
@@ -151,13 +149,11 @@ class CTEAssemblage:
         name: Symbol,
         schema: Optional[Symbol] = None,
         materialized: Optional[bool] = None,
-        external: bool = False,
     ) -> None:
         self.asmb = asmb
         self.name = name
         self.schema = schema
         self.materialized = materialized
-        self.external = external
 
 
 class TranslateContext:
@@ -204,20 +200,23 @@ class TranslateContext:
         else:
             alias = node_or_alias
 
-        self.aliases[alias] = self.aliases.get(alias, 0) + 1
-        return S(f"{alias}_{self.aliases[alias]}")
+        n = self.aliases.get(alias, 0) + 1
+        self.aliases[alias] = n
+        return S(f"{alias}_{n}")
 
 
 def translate_toplevel(node: SQLNode, ctx: TranslateContext) -> SQLClause:
-    """Translate the top-level SQLNode to produce a SQLClause representing the full query."""
+    """Translate the top-level SQLNode to produce a SQLClause representing the full query.
+
+    * The `translate` routine starts from the query node, and produces a SQLClause, while
+    any CTEs encountered are collected in the context.
+    * The CTEs are converted to a WITH clause, and prepended to the previous output.
+    """
     clause = translate(node, ctx)
     with_args: list[SQLClause] = []
 
-    # The CTE nodes encountered are collected in the translation `context`
+    # The CTE nodes encountered are collected in the translation context object
     for cte_asmb in ctx.cte_map.values():
-        if cte_asmb.external:
-            continue
-
         cols = [S(name) for name in cte_asmb.asmb.cols]
         if len(cols) == 0:
             cols.append(S("_"))
@@ -268,41 +267,38 @@ def translate(
     assert isinstance(node, SQLNode), f"expected SQLNode, got: {type(node)}"
     if node in ctx.subs:
         return ctx.subs[node]
+    elif isinstance(node, Box):
+        base: Assemblage = assemble(node, ctx)
+        return base.get_complete_select()
     else:
-        return translate_node(node, ctx)
+        return translate_scalar(node, ctx)
 
 
 @singledispatch
-def translate_node(node: SQLNode, ctx: TranslateContext) -> Optional[SQLClause]:
+def translate_scalar(node: SQLNode, ctx: TranslateContext) -> Optional[SQLClause]:
     """specific translations for each subtype of SQLNode"""
     raise NotImplementedError(
         f"Translation isn't implemented for node of type: {type(node)}"
     )
 
 
-@translate_node.register
+@translate_scalar.register
 def _(node: Agg, ctx: TranslateContext) -> SQLClause:
-    if str(node.name).upper() == "COUNT":
-        args = translate(node.args, ctx) if len(node.args) > 0 else [OP(S("*"))]
-        filter_ = translate(node.filter_, ctx)
-        return AGG(S("COUNT"), *args, distinct=node.distinct, filter_=filter_)
+    args: list[SQLClause] = []
+    if str(node._name).upper() == "COUNT" and len(node.args) == 0:
+        args = [OP(S("*"))]
     else:
         args = translate(node.args, ctx)
-        filter_ = translate(node.filter_, ctx)
-        return AGG(
-            S(str(node.name).upper()),
-            *args,
-            distinct=node.distinct,
-            filter_=filter_,
-        )
+    filter_ = translate(node.filter_, ctx)
+    return AGG(node._name, *args, distinct=node.distinct, filter_=filter_)
 
 
-@translate_node.register
+@translate_scalar.register
 def _(node: As, ctx: TranslateContext) -> Optional[SQLClause]:
     return translate(node.over, ctx)
 
 
-@translate_node.register
+@translate_scalar.register
 def _(node: IntBind, ctx: TranslateContext) -> Optional[SQLClause]:
     vars_p = ctx.vars_.copy()
     for name, i in node.label_map.items():
@@ -312,21 +308,15 @@ def _(node: IntBind, ctx: TranslateContext) -> Optional[SQLClause]:
         return translate(node.over, ctx)
 
 
-@translate_node.register
-def _(node: Box, ctx: TranslateContext) -> SQLClause:
-    base: Assemblage = assemble(node, ctx)
-    return base.get_complete_select()
-
-
 _FUNC_REPLACE = {
     "==": "=",
     "!=": "<>",
 }
 
 
-@translate_node.register
+@translate_scalar.register
 def _(node: Fun, ctx: TranslateContext) -> SQLClause:
-    fn = str(node.name).upper()
+    fn = str(node._name).upper()
 
     if fn in ("NOT", "LIKE", "EXISTS", "=", "==", "!="):
         name = S(_FUNC_REPLACE.get(fn, fn))
@@ -400,22 +390,22 @@ def _(node: Fun, ctx: TranslateContext) -> SQLClause:
         return OP(S(fn), *args)
 
 
-@translate_node.register
+@translate_scalar.register
 def _(node: Lit, ctx: TranslateContext) -> SQLClause:
     return LIT(node.val)
 
 
-@translate_node.register
+@translate_scalar.register
 def _(node: Sort, ctx: TranslateContext) -> SQLClause:
     return SORT(value=node.value, nulls=node.nulls, over=translate(node.over, ctx))
 
 
-@translate_node.register
+@translate_scalar.register
 def _(node: Var, ctx: TranslateContext) -> SQLClause:
-    if node.name in ctx.vars_:
-        return ctx.vars_[node.name]
+    if node._name in ctx.vars_:
+        return ctx.vars_[node._name]
     else:
-        return VAR(node.name)
+        return VAR(node._name)
 
 
 # -----------------------------------------------------------
@@ -533,7 +523,7 @@ def _(node: As, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
 def _(node: Define, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
     assert node.over is not None
     base = assemble(node.over, ctx)
-    if not any(isinstance(ref, Get) and ref.name in node.label_map for ref in refs):
+    if not any(isinstance(ref, Get) and ref._name in node.label_map for ref in refs):
         return base
 
     if not isinstance(base.clause, (SELECT, UNION)):
@@ -547,8 +537,8 @@ def _(node: Define, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
     translates: list[tuple[SQLNode, SQLClause]] = []
     tr_cache: ALIAS_TO_CLAUSE = dict()
     for ref in refs:
-        if isinstance(ref, Get) and ref.over is None and ref.name in node.label_map:
-            name = ref.name
+        if isinstance(ref, Get) and ref.over is None and ref._name in node.label_map:
+            name = ref._name
             if name not in tr_cache:
                 _define = node.args[node.label_map[name]]
                 with ctx.substitute_vars_n_subs(subs=subs):
@@ -571,14 +561,13 @@ def _(node: FromNothing, refs: list[SQLNode], ctx: TranslateContext) -> Assembla
 def _(node: FromReference, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
     cte_asmb = ctx.cte_map[node.over]
     asmb = cte_asmb.asmb.unwrap_repl()
-    clause = FROM(
-        over=ID(
-            name=cte_asmb.name,
-            over=ID(name=cte_asmb.schema) if cte_asmb.schema else None,
-        )
-    )
 
-    subs = asmb.get_make_subs(cte_asmb.name)
+    alias = ctx.allocate_alias(node.name)
+    schema = None if cte_asmb.schema is None else ID(name=cte_asmb.schema)
+    table = ID(name=cte_asmb.name, over=schema)
+    clause = FROM(AS(name=alias, over=table))
+
+    subs = asmb.get_make_subs(alias)
     translates = [(ref, subs[ref]) for ref in refs]
     repl, cols = translates_to_repl_n_cols(translates)
     return Assemblage(clause, cols=cols, repl=repl)
@@ -589,9 +578,11 @@ def _(node: FromTable, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage
     seen: set[Symbol] = set()
     for ref in refs:
         assert (
-            isinstance(ref, Get) and ref.over is None and ref.name in node.table.columns
+            isinstance(ref, Get)
+            and ref.over is None
+            and ref._name in node.table.columns
         )
-        seen.add(ref.name)
+        seen.add(ref._name)
     alias = ctx.allocate_alias(node.table.name)
 
     schema = None if node.table.schema is None else ID(name=node.table.schema)
@@ -606,7 +597,7 @@ def _(node: FromTable, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage
     repl: NODE_TO_ALIAS = dict()
     for ref in refs:
         if isinstance(ref, Get) and ref.over is None:
-            repl[ref] = ref.name
+            repl[ref] = ref._name
     return Assemblage(clause, cols=cols, repl=repl)
 
 
@@ -616,12 +607,12 @@ def _(node: FromValues, refs: list[SQLNode], ctx: TranslateContext) -> Assemblag
     column_set = set(columns)
     seen: set[Symbol] = set()
     for ref in refs:
-        assert isinstance(ref, Get) and ref.over is None and ref.name in column_set
-        seen.add(ref.name)
+        assert isinstance(ref, Get) and ref.over is None and ref._name in column_set
+        seen.add(ref._name)
 
     rows: list[tuple]
     column_aliases: list[Symbol]
-    # all/some/no columns are selected
+    # render the VALUES clause based on whether all/some/no columns are selected
     if len(seen) == len(columns):
         rows = node.source.data
         column_aliases = list(columns)
@@ -663,7 +654,7 @@ def _(node: FromValues, refs: list[SQLNode], ctx: TranslateContext) -> Assemblag
     repl: NODE_TO_ALIAS = dict()
     for ref in refs:
         if isinstance(ref, Get) and ref.over is None:
-            repl[ref] = ref.name
+            repl[ref] = ref._name
     return Assemblage(clause, cols=cols, repl=repl)
 
 
@@ -688,8 +679,8 @@ def _(node: Group, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
     translates: list[tuple[SQLNode, SQLClause]] = []
     for ref in refs:
         if isinstance(ref, Get) and ref.over is None:
-            assert ref.name in node.label_map
-            translates.append((ref, by[node.label_map[ref.name]]))
+            assert ref._name in node.label_map
+            translates.append((ref, by[node.label_map[ref._name]]))
         elif isinstance(ref, Agg) and ref.over is None:
             with ctx.substitute_vars_n_subs(subs=subs):
                 translates.append((ref, translate(ref, ctx)))
@@ -859,6 +850,7 @@ def _(node: Knot, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
 
     clause = FROM(over=ID(union_alias))
     subs = union.get_make_subs(union_alias)
+
     translates = [(ref, subs[ref]) for ref in refs]
     repl, cols = translates_to_repl_n_cols(translates)
     return Assemblage(clause, cols=cols, repl=repl)
@@ -967,7 +959,7 @@ def _(node: Select, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
     repl: NODE_TO_ALIAS = dict()
     for ref in refs:
         assert isinstance(ref, Get) and ref.over is None
-        repl[ref] = ref.name
+        repl[ref] = ref._name
     return Assemblage(clause, cols=cols, repl=repl)
 
 
@@ -1035,20 +1027,3 @@ def _(node: With, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
             asmb, name=alias, materialized=node.materialized
         )
     return assemble(node.over, ctx)
-
-
-@assemble.register
-def _(node: WithExternal, refs: list[SQLNode], ctx: TranslateContext) -> Assemblage:
-    for arg in node.args:
-        asmb = assemble(arg, ctx)
-        assert isinstance(arg, Box)
-        table_name = arg.typ.name
-        table_cols = [c for c in asmb.cols]
-        if len(table_cols) == 0:
-            table_cols.append(S("_"))
-
-        table = SQLTable(name=table_name, schema=node.schema, columns=table_cols)
-        if node.handler is not None:
-            node.handler(table, asmb.get_complete_select())
-        ctx.cte_map[arg] = CTEAssemblage(asmb, name=table.name, external=True)
-    return assemble(node.over, ctx)
